package cz.muni.fi.pa165.json;

import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.core.client.JsonUtils;
import com.google.gwt.http.client.*;
import com.google.gwt.json.client.JSONObject;
import com.google.gwt.user.client.Window;
import cz.muni.fi.pa165.model.Error;

import java.util.ArrayList;
import java.util.List;

/**
 * Generic class for calling various methods from BOOK APP API.
 *
 * @author Pavel Zlámal <zlamal@cesnet.cz>
 */
public class ApiCaller {

	private static final String BASE_URL = "http://localhost:8080/springmvc-json-server/mvc/book/";

	public static Request call(RequestBuilder.Method httpMethod, String apiMethod, final ApiCallerEvent event) {
		return call(httpMethod, apiMethod, null, event);
	}

	public static Request call(RequestBuilder.Method httpMethod, String apiMethod, JSONObject jsonObject, final ApiCallerEvent event) {

		RequestBuilder builder = new RequestBuilder(httpMethod, BASE_URL+apiMethod);

		try {

			builder.setHeader("Content-Type", "application/json");

			if (event != null) event.onStart();

			return builder.sendRequest((jsonObject != null) ? jsonObject.toString() : "", new RequestCallback() {
				@Override
				public void onResponseReceived(Request request, Response response) {

					if (response.getStatusCode() == 200) {

						if (event != null) event.onFinished(parseJson(response.getText()));

					} else if (response.getStatusCode() == 422) {

						// expected object is Error
						if (event != null) event.onError((Error)parseJson(response.getText()));

					}

				}

				@Override
				public void onError(Request request, Throwable exception) {

					// TODO
					Window.alert(exception.toString());

				}
			});

		} catch (RequestException ex) {

			// TODO
			Window.alert(ex.toString());

		}

		return null;

	}

	/*
	 * Takes in a JSON String and evals it.
	 *
	 * @param JSON String that you trust
	 * @return JavaScriptObject that you can cast to an Overlay Type
	 */
	public static <T extends JavaScriptObject> T parseJson(String jsonStr) {
		return JsonUtils.safeEval(jsonStr);
	}

	public static final native <T extends JavaScriptObject> JsArray<T> toJsArray(JavaScriptObject jso) /*-{
		return jso;
	}-*/;

	public static final <T extends JavaScriptObject> List<T> toList(JavaScriptObject jso) {
		List<T> list = new ArrayList<T>();
		JsArray<T> array = toJsArray(jso);
		for (int i=0; i<array.length(); i++) {
			list.add(array.get(i));
		}
		return list;
	}

}