package cz.muni.fi.pa165.model;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;

/**
 * Memory-based book repository.
 *
 * @author Martin Kuba makub@ics.muni.cz
 */
@Repository("bookService")
public class BookServiceImpl implements BookService {

    final static Logger log = LoggerFactory.getLogger(BookServiceImpl.class);

    private List<Book> books = new ArrayList<>(Arrays.asList(
            new Book(1, "Karel May", "Vinnetou", 100, false, Book.Color.BLUE),
            new Book(2, "Božena Němcová", "Babička", 200, true, Book.Color.ORANGE),
            new Book(3, "Isaac Asimov", "I, robot", 500, true, Book.Color.GREEN)
    ));

    private int counter = 10;

    @Override
    public void createBook(Book book) {
        book.setId(++counter);
        log.debug("adding book {}", book);
        books.add(book);
    }

    @Override
    public Book getBookById(int id) {
        log.debug("getting book id={}", id);
        for (Book book : books) {
            if (book.getId() == id) return book;
        }
        return null;
    }

    @Override
    public List<Book> getAllBooks() {
        log.debug("getting all books");
        return new ArrayList<>(books);
    }

    @Override
    public void updateBook(Book book) {
        log.debug("updating book {}", book);
        ListIterator<Book> li = books.listIterator();
        while (li.hasNext()) {
            Book b1 = li.next();
            if (b1.getId() == book.getId()) {
                li.set(book);
                return;
            }
        }
    }

    @Override
    public void deleteBook(Book book) {
        log.debug("deleting book {}", book);
        books.remove(book);
    }

    @Override
    public List<Book> findBookNamesBySubstring(String substr) {
        ArrayList<Book> foundBooks = new ArrayList<>();
        for (Book book : books) {
            if (book.getName().contains(substr)) {
                foundBooks.add(book);
            }
        }
        return foundBooks;
    }

}
